package controller

import (
	"encoding/json"
	"strings"

	cm_omc "be.ems/features/cm/omc"
	"be.ems/src/framework/i18n"
	"be.ems/src/framework/utils/ctx"
	"be.ems/src/framework/utils/parse"
	"be.ems/src/framework/vo/result"
	neFetchlink "be.ems/src/modules/network_element/fetch_link"
	"be.ems/src/modules/network_element/model"
	neService "be.ems/src/modules/network_element/service"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

// NewNeConfig 网元参数配置 实例化控制层
var NewNeConfig = &NeConfigController{
	neConfigService: neService.NewNeConfig,
	neInfoService:   neService.NewNeInfo,
}

// 网元参数配置
//
// PATH /config
type NeConfigController struct {
	neConfigService *neService.NeConfig // 网元参数配置可用属性值服务
	neInfoService   *neService.NeInfo   // 网元信息服务
}

// 网元参数配置可用属性值列表
//
// GET /list
func (s *NeConfigController) List(c *gin.Context) {
	querys := ctx.QueryMap(c)
	data := s.neConfigService.SelectPage(querys)

	c.JSON(200, result.Ok(data))
}

// 网元参数配置可用属性值信息
//
// GET /info/:id
func (s *NeConfigController) Info(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	id := c.Param("id")
	if id == "" {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	data := s.neConfigService.SelectById(id)
	if data.ID != id {
		// 没有可访问参数配置数据！
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "neConfig.noData")))
		return
	}

	// 将字符串转json数据
	if err := json.Unmarshal([]byte(data.ParamJson), &data.ParamData); err != nil {
		c.JSON(400, result.CodeMsg(400, err.Error()))
		return
	}
	c.JSON(200, result.OkData(data))
}

// 网元参数配置可用属性值新增
//
// POST /
func (s *NeConfigController) Add(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var body model.NeConfig
	if err := c.ShouldBindBodyWith(&body, binding.JSON); err != nil {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	// 将json数据转字符串存储
	paramDataByte, err := json.Marshal(body.ParamData)
	if err != nil {
		c.JSON(400, result.CodeMsg(400, err.Error()))
		return
	}
	body.ParamJson = string(paramDataByte)

	insertId := s.neConfigService.Insert(body)
	if insertId != "" {
		c.JSON(200, result.Ok(nil))
		return
	}
	c.JSON(200, result.Err(nil))
}

// 网元参数配置可用属性值修改
//
// PUT /
func (s *NeConfigController) Edit(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var body model.NeConfig
	err := c.ShouldBindBodyWith(&body, binding.JSON)
	if err != nil || body.ID == "" {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	// 检查是否存在
	data := s.neConfigService.SelectById(body.ID)
	if data.ID != body.ID {
		// 没有可访问主机命令数据！
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "neConfig.noData")))
		return
	}

	// 将json数据转字符串存储
	paramDataByte, err := json.Marshal(body.ParamData)
	if err != nil {
		c.JSON(400, result.CodeMsg(400, err.Error()))
		return
	}
	body.ParamJson = string(paramDataByte)

	rows := s.neConfigService.Update(body)
	if rows > 0 {
		c.JSON(200, result.Ok(nil))
		return
	}
	c.JSON(200, result.Err(nil))
}

// 网元参数配置可用属性值删除
//
// DELETE /
func (s *NeConfigController) Remove(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	id, okId := c.GetQuery("id")
	if id == "" || !okId {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}
	// 处理字符转id数组后去重
	idsArr := strings.Split(id, ",")
	uniqueIDs := parse.RemoveDuplicates(idsArr)
	if len(uniqueIDs) <= 0 {
		c.JSON(200, result.Err(nil))
		return
	}
	rows, err := s.neConfigService.DeleteByIds(uniqueIDs)
	if err != nil {
		c.JSON(200, result.ErrMsg(i18n.TKey(language, err.Error())))
		return
	}
	msg := i18n.TTemplate(language, "app.common.deleteSuccess", map[string]any{"num": rows})
	c.JSON(200, result.OkMsg(msg))
}

// 网元参数配置可用属性值列表指定网元类型全部无分页
//
// GET /list/:neType
func (s *NeConfigController) ListByNeType(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	neType := c.Param("neType")
	if neType == "" {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}
	data := s.neConfigService.SelectNeConfigByNeType(neType)
	c.JSON(200, result.OkData(data))
}

// 网元参数配置数据信息
//
// GET /data
func (s *NeConfigController) DataInfo(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var query struct {
		NeType    string `form:"neType" binding:"required"`    // 网元类型
		NeId      string `form:"neId" binding:"required"`      // 网元ID
		ParamName string `form:"paramName" binding:"required"` // 可用属性
	}
	if err := c.ShouldBindQuery(&query); err != nil {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	neInfo := s.neInfoService.SelectNeInfoByNeTypeAndNeID(query.NeType, query.NeId)
	if neInfo.NeId != query.NeId || neInfo.IP == "" {
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "app.common.noNEInfo")))
		return
	}

	if query.NeType == "OMC" {
		var o *cm_omc.ConfigOMC
		resData, err := o.Query(query.ParamName)
		if err != nil {
			c.JSON(200, result.ErrMsg(err.Error()))
			return
		}
		c.JSON(200, result.OkData(resData))
		return
	} else {
		// 网元直连
		resData, err := neFetchlink.NeConfigInfo(neInfo, query.ParamName)
		if err != nil {
			c.JSON(200, result.ErrMsg(err.Error()))
			return
		}

		c.JSON(200, result.Ok(resData))
	}
}

// 网元参数配置数据修改
//
// PUT /data
func (s *NeConfigController) DataEdit(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var body struct {
		NeType    string         `json:"neType" binding:"required"` // 网元类型
		NeId      string         `json:"neId" binding:"required"`   // 网元ID
		ParamName string         `json:"paramName" binding:"required"`
		ParamData map[string]any `json:"paramData" binding:"required"`
		Loc       string         `json:"loc"` // 仅array使用与数据对象内index一致,有多层时划分嵌套层(index/subParamName/index)
	}
	if err := c.ShouldBindBodyWith(&body, binding.JSON); err != nil {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	neInfo := s.neInfoService.SelectNeInfoByNeTypeAndNeID(body.NeType, body.NeId)
	if neInfo.NeId != body.NeId || neInfo.IP == "" {
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "app.common.noNEInfo")))
		return
	}
	if body.NeType == "OMC" {
		var o *cm_omc.ConfigOMC
		resData, err := o.Modify(body.ParamName, body.ParamData)
		if err != nil {
			c.JSON(200, result.ErrMsg(err.Error()))
			return
		}
		c.JSON(200, result.OkData(resData))
		return
	} else {
		// 网元直连
		resData, err := neFetchlink.NeConfigUpdate(neInfo, body.ParamName, body.Loc, body.ParamData)
		if err != nil {
			c.JSON(200, result.ErrMsg(err.Error()))
			return
		}
		c.JSON(200, result.OkData(resData))
	}
}

// 网元参数配置数据新增（array）
//
// POST /data
func (s *NeConfigController) DataAdd(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var body struct {
		NeType    string         `json:"neType" binding:"required"`    // 网元类型
		NeId      string         `json:"neId" binding:"required"`      // 网元ID
		ParamName string         `json:"paramName" binding:"required"` // 根据配置可选值
		ParamData map[string]any `json:"paramData" binding:"required"` // 数据对象
		Loc       string         `json:"loc" binding:"required"`       // 与数据对象内index一致,有多层时划分嵌套层(index/subParamName/index)
	}
	if err := c.ShouldBindBodyWith(&body, binding.JSON); err != nil {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	// 检查是否array
	info := s.neConfigService.SelectNeConfigByNeTypeAndParamName(body.NeType, body.ParamName)
	if info.ParamType != "array" {
		c.JSON(400, result.CodeMsg(400, "this attribute does not support adding"))
		return
	}
	// 必须含有index
	_, idxOk := body.ParamData["index"]
	if info.ParamType == "array" && !idxOk {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	neInfo := s.neInfoService.SelectNeInfoByNeTypeAndNeID(body.NeType, body.NeId)
	if neInfo.NeId != body.NeId || neInfo.IP == "" {
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "app.common.noNEInfo")))
		return
	}

	// 网元直连
	resData, err := neFetchlink.NeConfigInstall(neInfo, body.ParamName, body.Loc, body.ParamData)
	if err != nil {
		c.JSON(200, result.ErrMsg(err.Error()))
		return
	}
	c.JSON(200, result.OkData(resData))
}

// 网元参数配置数据删除（array）
//
// DELETE /data
func (s *NeConfigController) DataRemove(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var query struct {
		NeType    string `form:"neType" binding:"required"` // 网元类型
		NeId      string `form:"neId" binding:"required"`   // 网元ID
		ParamName string `form:"paramName" binding:"required"`
		Loc       string `form:"loc" binding:"required"` // 与数据对象内index一致,有多层时划分嵌套层(index/subParamName/index)
	}
	if err := c.ShouldBindQuery(&query); err != nil {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	// 检查是否array
	info := s.neConfigService.SelectNeConfigByNeTypeAndParamName(query.NeType, query.ParamName)
	if info.ParamType != "array" {
		c.JSON(400, result.CodeMsg(400, "this attribute does not support adding"))
		return
	}

	neInfo := s.neInfoService.SelectNeInfoByNeTypeAndNeID(query.NeType, query.NeId)
	if neInfo.NeId != query.NeId || neInfo.IP == "" {
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "app.common.noNEInfo")))
		return
	}

	// 网元直连
	resData, err := neFetchlink.NeConfigDelete(neInfo, query.ParamName, query.Loc)
	if err != nil {
		c.JSON(200, result.ErrMsg(err.Error()))
		return
	}
	c.JSON(200, result.OkData(resData))
}
